# Migration guide 7.0 -> 8.0

* PrimeFaces is now only compatible with Java8+
* Removed long time deprecated image attribute on buttons
* Facelet function `p:component` has been removed in favor of: `p:resolveFirstComponentWithId('myId', view).clientId`
* Facelet function `p:widgetVar` have been removed in favor of: `p:resolveWidgetVar('myTable', view)`
* TextEditor for security reasons (https://github.com/primefaces/primefaces/issues/5163) now requires the OWASP Sanitizer library by default but you can choose to opt-out of using the library by setting the attribute `secure="false"`.
* InputMask for security reasons (https://github.com/primefaces/primefaces/issues/5105) validates the mask used on the server side. If you would like to disable that set validateMask="false".  Especially if you use a custom mask with `$.mask.definitions` for example: https://github.com/primefaces/primefaces/issues/3234#issuecomment-361221532
* `p:watermark`has been reimplemented with a simple HTML placeholder attribute. Therefore `PrimeFaces.cleanWatermarks()` and `PrimeFaces.showWatermarks()` has been removed.
* `org.primefaces.json` has been moved to `org.primefaces.shaded.json`
* Many events (eg SelectEvent, UnselectEvent, RowEditEvent, etc.) now use generics
* Cropper: change libraries from JCrop to Cropper.js
* PanelGrid: new layout="flex" based on PrimeFlex/FlexGrid

## Calendar / DatePicker
  * Without a value-binding it converts OOTB to java `java.time.LocalDate`. (up to PrimeFaces 7.0:  `java.util.Date`) This may be relevant when you use Calendar or DatePicker as a filter for eg DataTable. You can add something like `<f:convertDateTime type="date" />` to your Calendar or DatePicker to get a java.util.Date.
  * built-in support for `java.time.LocalDate`, `java.time.LocalDateTime` and `java.time.LocalTime` (no converter needed anymore)

## StreamedContent API
  * `DefaultStreamedContent`: Constructors and setters have been deprecated in favor of `DefaultStreamedContent.Builder`
  * `ByteArrayContent` has been removed in favor of `DefaultStreamedContent.Builder#stream(() -> new ByteArrayInputStream(...))`
  * `LazyDefaultStreamedContent` has been removed, instead use `DefaultStreamedContent` with a `Supplier` which will always "lazy-creates" the InputStream: `DefaultStreamedContent.Builder#stream(() -> new FileInputStream(...))`

## DataTable
  * Methods `PrimeFaces#clearTableStates()` and `PrimeFaces#clearDataListStates()` have been deprecated in favor of `PrimeFaces#multiViewState()`
  * DataTable: `getFilters` / `setFilters` has been removed in favor of `getFilterBy` / `setFilterBy`
  * `LazyDataModel#load` signatures has been changed; sortBy is now a Map instead of List, filterBy Map value is now a `FilterMeta` object instead the filter value.

## Menu
  * `DynamicMenuModel` has been removed, use `BaseMenuModel` instead.
  * Fluent builders: instanciate menu items using builders (e.g `DefaultMenuItem#builder()`)
  * `MenuModel#addElement()` has been deprecated, add element using `MenuModel#getElements()#add()` instead.

## FileUpload
  * Models have been moved from `org.primefaces.model` to `org.primefaces.model.file`
  * `UploadedFile#getContents` has been renamed to `getContent`
  * `UploadedFile#getInputstream` has been renamed to `getInputStream`
  * `FileUpload#fileUploadListener` has been renamed into `listener`
  * In case of using multiple feature in a simple mode, use `UploadedFiles` model to get all files in a single request.

## DataExporter
  * `org.primefaces.component.export.Exporter` interface allows you to implement your own exporter using `exporter` attribute
  * Removal of `repeat` attribute: used as a workaround in case of tables contained in an iterator component _(e.g `ui:repeat`, `p:tabView` etc.)_. No longer necessary, instead, define every ids component you wish to export (without specifying component index)
  * Apache POI minimum version 4.1.1 to use Excel export

## InputNumber
  * upgraded using [Autonumeric](http://autonumeric.org/)
  * `emptyValue`: `sign` is now `always`. Default value is `focus`
  * `symbolPosition`: `prefix` is now `p`, `suffix` is now `s` default is `p`
  * `decimalPlaces`: now defaults to 2. If value is Integer/Long/Short number defaults to 0.

## TimeLine
  * updated to the newest [vis](https://visjs.org/) timeline version.
  * Fluent builders: instanciate timeline events using builders (e.g `TimelineEvent#builder()`)
  * `TimelineEvent` uses `java.time.LocalDateTime` instead of `java.util.Date`
  * `TimelineEvent` has now 3 more properties to fine tunning editable: editableTime, editableGroup and editableRemove.
  * `TimelineGroup` has now styleClass property similar to the one in TimelineEvent.
  * Properties deprecated in favor of others: `axisOnTop`, `groupsChangeable`, `groupsWidth`, `groupMinHeight`, `snapEvents`, `timeChangeable`.
  * Properties deprecated with NO replacement: `dragAreaWith` (use css classes instead), `unselectable`, `groupsOnRight`, `animate`, `animateZoom`, `showButtonNew`, `showNavigation`, `browserTimeZone`.
  * New properties: `maxHeight`, `orientationAxis`, `orientationItem`, `editableAdd`, `editableTime`, `editableGroup`, `editableRemove`, `editableOverrideItems`, `eventStyle `, `eventHorizontalMargin`, `eventVerticalMargin`, `groupStyle`, `snap`, `clickToUse`, `clientTimeZone`.
  * New `extender` property to specify a javascript function to extend configuration of vis.js timeline component similar to extender property in `<p:chart />` component.

## Schedule
  * upgraded to FullCalendar 4.x
  * Fluent builders: instanciate schedule events using builders (e.g `DefaultScheduleEvent#builder()`)
  * `ScheduleEvent` uses `java.time.LocalDateTime` instead of `java.util.Date`
  * Removed deprecated attributes
  * Localization comes with FullCalendar: does not require/use PF-localization. Just set `locale="fr"` for example on the Schedule.
  * New view names (`dayGridMonth`, `dayGridWeek`, `dayGridDay`, `timeGridWeek`, `timeGridDay`, `listYear`,` listMonth`, `listWeek`, `listDay`): old ones are translated by Schedule component
  * `ScheduleEntryMoveEvent`: new properties `yearDelta` and `monthDelta`
  * `ScheduleEntryResizeEvent`: new properties `yearDelta` and `monthDelta`: splitted properties between start and end (up to PF 7.0 only end)
  * Property `slotLabelFormat` was removed (only available in commercial version of FullCalendar - see https://fullcalendar.io/docs/slotLabelFormat)
  * `ScheduleEvent`: new properties `groupId` and `overlapAllowed`

